"""Define constants used in the SmartApp."""

__title__ = "pysmartapp"
__version__ = "0.3.0"

LIFECYCLE_PING = 'PING'
LIFECYCLE_CONFIG = 'CONFIGURATION'
LIFECYCLE_CONFIG_INIT = 'INITIALIZE'
LIFECYCLE_CONFIG_PAGE = 'PAGE'
LIFECYCLE_INSTALL = 'INSTALL'
LIFECYCLE_UPDATE = 'UPDATE'
LIFECYCLE_EVENT = 'EVENT'
LIFECYCLE_OAUTH_CALLBACK = 'OAUTH_CALLBACK'
LIFECYCLE_UNINSTALL = 'UNINSTALL'
EVENT_TYPE_DEVICE = 'DEVICE_EVENT'
EVENT_TYPE_TIMER = 'TIMER_EVENT'
SETTINGS_APP_ID = 'appId'
